/*
 *    Example program for the Allegro library, by Shawn Hargreaves.
 *    Modified by Andrei Ellman to include color-bars and use HLS as well
 *
 *    This program shows how to convert colors between the different
 *    color-space representations.
 */


#include "allegro.h"


#define DIALOG_NUM_SLIDERS   9
#define DIALOG_FIRST_COLOR_BAR   18
#define DIALOG_COLOR_BOX   27

/* slider types (R, G, B, and H, S, V) */
#define S_R    0
#define S_G    1
#define S_B    2
#define S_H    3
#define S_SV   4
#define S_V    5
#define S_L    6
#define S_SL   7
#define S_HL   8



/* the current color values */
static int colors[DIALOG_NUM_SLIDERS] =
{
   255,     /* red */
   255,     /* green */
   255,     /* blue */
   0,       /* hue */
   0,       /* saturation (sv) */
   255,     /* value */
   255,     /* lightness */
   0,       /* saturation (sl) */
   0        /* hue (hl) */
};


/* The bitmaps containing the color-bars */
static BITMAP *color_bar_bitmap[DIALOG_NUM_SLIDERS];

/* RGB -> color mapping table. Not needed, but speeds things up */
/*
RGB_MAP rgb_table;
*/


int my_slider_proc(int msg, DIALOG *d, int c);
int update_color_value(void *dp3, int val);


DIALOG the_dlg[] =
{
   /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1)  (d2)  (dp)     (dp2)          (dp3) */
   { my_slider_proc,    32,   16,   256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_R]  },
   { my_slider_proc,    32,   64,   256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_G]  },
   { my_slider_proc,    32,   112,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_B]  },
   { my_slider_proc,    16,   336,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_H]  },
   { my_slider_proc,    16,   384,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_SV] },
   { my_slider_proc,    16,   432,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_V]  },
   { my_slider_proc,    368,  432,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_L]  },
   { my_slider_proc,    368,  384,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_SL] },
   { my_slider_proc,    368,  336,  256,  16,   0,    255,  0,    0,       255,  0,    NULL,    update_color_value,  &colors[S_HL] },
   { d_text_proc,       308,  22,   0,    0,    0,    255,  0,    0,       0,    0,    "R",     NULL,          NULL          },
   { d_text_proc,       308,  70,   0,    0,    0,    255,  0,    0,       0,    0,    "G",     NULL,          NULL          },
   { d_text_proc,       308,  118,  0,    0,    0,    255,  0,    0,       0,    0,    "B",     NULL,          NULL          },
   { d_text_proc,       292,  342,  0,    0,    0,    255,  0,    0,       0,    0,    "H",     NULL,          NULL          },
   { d_text_proc,       292,  390,  0,    0,    0,    255,  0,    0,       0,    0,    "S",     NULL,          NULL          },
   { d_text_proc,       292,  438,  0,    0,    0,    255,  0,    0,       0,    0,    "V",     NULL,          NULL          },
   { d_text_proc,       342,  438,  0,    0,    0,    255,  0,    0,       0,    0,    "L",     NULL,          NULL          },
   { d_text_proc,       342,  390,  0,    0,    0,    255,  0,    0,       0,    0,    "S",     NULL,          NULL          },
   { d_text_proc,       342,  342,  0,    0,    0,    255,  0,    0,       0,    0,    "H",     NULL,          NULL          },
   { d_bitmap_proc,     32,   32,   256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     32,   80,   256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     32,   128,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     16,   352,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     16,   400,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     16,   448,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     368,  448,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     368,  400,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_bitmap_proc,     368,  352,  256,  16,   0,    255,  0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { d_box_proc,        170,  170,  300,  140,  0,    0,    0,    0,       0,    0,    NULL,    NULL,          NULL          },
   { NULL,              0,    0,    0,    0,    0,    0,    0,    0,       0,    0,    NULL,    NULL,          NULL          }
};















/****************************************
 * Start of extra color-space functions *
 ****************************************/

// Note: These are currently un-optimised.


#include <math.h>
#include <float.h>




/* hls_to_rgb:
 *  Converts from HLS colorspace to RGB values.
 *  Taken From Foley & VanDam Fig. 13.37 with a few modifications.
 */

/* First, a helper function for working out a value for a color chanel */
INLINE float _hls_to_rgb_value (float n1, float n2, float hx)
{
   if (hx >= 6.0f)
      hx -= 6.0f;

   if (hx < 1.0f)
      return n1 + (n2-n1)*hx;
   else if (hx < 3.0f)
      return n2;
   else if (hx < 4.0f)
      return n1 + (n2-n1)*(4.0f-hx);
   else
      return n1;
}

/* The function proper */
void hls_to_rgb (float h, float l, float s, int *r, int *g, int *b)
{
   float m1, m2;

   /* Work out min and max of the r,g,b sliders */
   m2 = (l<=0.5f) ? (l+(l*s)) : (l+s - (l*s));
   m1 = 2.0f * l - m2; /* L is average of m1 and m2. */

   if (s < FLT_EPSILON) {   /* greyscale */
      *r = *g = *b = (unsigned char)(l * 255.0f);
   } else {
      h = fmod(h, 360.0f) / 60.0f;
      if (h < 0.0f) {
         h += 6.0f;
      }

      *r = (int)(_hls_to_rgb_value(m1, m2, h+2.0f) * 255.0f + 0.5f);
      *g = (int)(_hls_to_rgb_value(m1, m2, h) * 255.0f + 0.5f);
      *b = (int)(_hls_to_rgb_value(m1, m2, h+4.0f) * 255.0f + 0.5f);
   }
}   /* End of hls_to_rgb */


/* rgb_to_hls:
 *  Converts an RGB value into the HLS colorspace.
 *  Taken From Foley & VanDam Fig. 13.36 with a few modifications.
 */
void rgb_to_hls(int r, int g, int b, float *h, float *l, float *s)
{
   float min, max, delta, rc, gc, bc;

   rc = (float)r / 255.0f;
   gc = (float)g / 255.0f;
   bc = (float)b / 255.0f;

   max = MAX(rc, MAX(gc, bc));
   min = MIN(rc, MIN(gc, bc));

   *l = (max+min)/2.0f;   /* L is the average of maximum and minimum r,g,b */

   delta = max - min;


   // See the following thread on Allegro.cc for why I'm not using '==0.0f' here
   // http://www.allegro.cc/forums/view_thread.php?_id=269409&request=1053865245&
   if (delta<FLT_EPSILON) {
      /* Somewhere along the centre axis of the double-hex-cone (s==0) */
      *s = 0.0f;        /* color has no saturation */
      *h = 0.0f;        /* color has no hue */

   } else {

      /* Calculate Saturation */
      *s = (*l<=0.5f) ? (delta/(max+min)) : (delta/(2.0f-(max+min)));


      /* Calculate Hue */
      if (rc == max)
         *h = (gc-bc) / delta;  /* Resulting color between yellow & magenta */
      else if (gc == max)
         *h = 2.0f + (bc-rc) / delta;  /* Resulting color between cyan & yellow */
      else /* if (bc == max) */
         *h = 4.0f + (rc-gc) / delta;  /* Resulting color between magenta & cyan */

      if (*h < 0.0f)
         *h += 6.0f;   /* Make sure hue is non-negative */

      *h *= 60.0f;
   }

}   /* End of rgb_to_hls */






/* sv_to_ls:
 *  Converts a SV pair in HSV colorspace to a LS pair in HLS colorspace.
 */
void sv_to_ls(float sv, float v, float *l, float *sl)
{
   float vmin, delta;

   // See the following thread on Allegro.cc for why I'm not using '==0.0f' here
   // http://www.allegro.cc/forums/view_thread.php?_id=269409&request=1053865245&
   if(sv<FLT_EPSILON) {
      /* Somewhere along the centre axis of the hex-cone (sv==0) */
      *l=v;
      *sl=0.0f;
   }
   else if (v<FLT_EPSILON) {
      /* At the bottom-apex of the hex-cone (v==0) */
      *l=0.0f;
      *sl=0.0f;
   }
   else
   {
      vmin = v * (1.0f - sv); // v + vmin = v + v * (1 - sv) = v * (1 + 1 - sv) = v * (2 - sv) = v*2 - v*sv
      delta = v - vmin;   // v - vmin = v - v * (1 - sv) = v * (1 - 1 - sv) = v * sv

      *l = (v+vmin)/2.0f;   /* L is the average of maximum and minimum r,g,b */
      *sl = (*l<=0.5f) ? (delta/(v+vmin)) : (delta/(2.0f-(v+vmin)));   // v+vmin = 2*l
   }

}


/* ls_to_sv:
 *  Converts a SV pair in HSV colorspace to a LS pair in HLS colorspace.
 */
void ls_to_sv(float l, float sl, float *sv, float *v)
{
   float vmin, delta;


   if(sl<FLT_EPSILON) {
      /* Somewhere along the centre axis of the hex-cone (sl==0) */
      *v = l;
      *sv = 0.0f;
   }
   else if(l<FLT_EPSILON) {
      /* At the bottom-apex of the double hex-cone (l==0) */
      *v = 0.0f;
      *sv = 0.0f;
   }
   else if(l>1.0f-FLT_EPSILON) {
      /* At the top-apex of the double hex-cone (l==1) */
      *v = 1.0f;
      *sv = 0.0f;
   }
   else
   {
      *v = (l<=0.5f) ? (l+(l*sl)) : (l+sl - (l*sl));

      vmin = 2.0f * l - *v; /* l is average of vmin and v. */
      delta = *v - vmin;

      *sv = delta / *v;
   }

}




/**************************************
 * End of extra color-space functions *
 **************************************/












/* Helper that updates the color rectangle */
void update_color_rectangle(int r, int g, int b)
{
   the_dlg[DIALOG_COLOR_BOX].bg = makecol(r, g, b);
}


/* Helper that updates all six color-bar bitmaps according to the given RGB and HSV values */
void update_color_bars(int r, int g, int b, float h, float s, float v, float l, float sl)
{
   int i;
   int hr,hg,hb;   /* Temp RGB values for drawing the HSV and HLS sliders */


   for(i=0;i<256;i++) {
      vline(color_bar_bitmap[S_R], i, 0, 15, makecol32(i, g, b));   /* Red color-bar */

      vline(color_bar_bitmap[S_G], i, 0, 15, makecol32(r, i, b));   /* Green color-bar */

      vline(color_bar_bitmap[S_B], i, 0, 15, makecol32(r, g, i));   /* Blue color-bar */


	  /* The two hue color-bars are equivalent */
      hsv_to_rgb(i*360.0f/255.0f, s, v, &hr, &hg, &hb);
      vline(color_bar_bitmap[S_H], i, 0, 15, makecol32(hr, hg, hb));   /* Hue color-bar (HSV) */
      vline(color_bar_bitmap[S_HL], i, 0, 15, makecol32(hr, hg, hb));   /* Hue color-bar (HLS) */


      hsv_to_rgb(h, i/255.0f, v, &hr, &hg, &hb);
      vline(color_bar_bitmap[S_SV], i, 0, 15, makecol32(hr, hg, hb));   /* Saturation color-bar */

      hsv_to_rgb(h, s, i/255.0f, &hr, &hg, &hb);
      vline(color_bar_bitmap[S_V], i, 0, 15, makecol32(hr, hg, hb));   /* Value color-bar */


      hls_to_rgb(h, i/255.0f, sl, &hr, &hg, &hb);
      vline(color_bar_bitmap[S_L], i, 0, 15, makecol32(hr, hg, hb));   /* Lightness color-bar */

      hls_to_rgb(h, l, i/255.0f, &hr, &hg, &hb);
      vline(color_bar_bitmap[S_SL], i, 0, 15, makecol32(hr, hg, hb));   /* Saturation color-bar */
   }

}


/* helper for reacting to the changing one of the sliders */
int update_color_value(void *dp3, int val)
{
   /* 'val' is the value of the slider's position (0-255), 'type' is which slider was changed */
   int type = ((unsigned long)dp3 - (unsigned long)colors) / sizeof(colors[0]);
   int r, g, b;
   float h;	/* Hue is the same in HSV and HLS color-spaces so we only need one h variable */
   float s, v;
   float l, sl;

   if (colors[type] != val) {
      colors[type] = val;

      if ((type == S_R) || (type == S_G) || (type == S_B)) {
    /* The slider that's changed is either R, G, or B. Convert RGB color to HSV and HLS */
    r = colors[S_R];
    g = colors[S_G];
    b = colors[S_B];


    rgb_to_hsv(r, g, b, &h, &s, &v);
    rgb_to_hls(r, g, b, &h, &l, &sl);


    colors[S_H] = h * 255.0f / 360.0f + 0.5f;
    colors[S_SV] = s * 255.0f + 0.5f;
    colors[S_V] = v * 255.0f + 0.5f;

    colors[S_L] = l * 255.0f + 0.5f;
    colors[S_SL] = sl * 255.0f + 0.5f;
    colors[S_HL] = h * 255.0f / 360.0f + 0.5f;

      }
      else if ((type == S_H) || (type == S_SV) || (type == S_V)) {
    /* The slider that's changed is either H, S, or V. Convert HSV color to RGB and HLS */
    h = colors[S_H] * 360.0f / 255.0f;
    s = colors[S_SV] / 255.0f;
    v = colors[S_V] / 255.0f;

    hsv_to_rgb(h, s, v, &r, &g, &b);
    sv_to_ls(s,v, &l,&sl);

    /*
    // Used for testing purposes
    rgb_to_hls(r, g, b, &h, &l, &sl);
    {
       float sl_l, sl_s;
       float _sv, _v;
       text_mode(makecol(255,255,255));
       sv_to_ls(s,v, &sl_l, &sl_s);
       textprintf(screen,font,352,40,makecol(255,192,255),"%f  %f   ",l,sl);
       textprintf(screen,font,352,48,makecol(255,0,255),"%f  %f   ",sl_l,sl_s);

       ls_to_sv(sl_l,sl_s, &_sv, &_v);	// Test to see if inverse brings back same results
       textprintf(screen,font,352,60,makecol(255,192,255),"%f  %f   ",s,v);
       textprintf(screen,font,352,68,makecol(255,224,255),"%f  %f   ",_sv,_v);
    }
    */




    colors[S_R] = r;
    colors[S_G] = g;
    colors[S_B] = b;

    colors[S_L] = l * 255.0f + 0.5f;
    colors[S_SL] = sl * 255.0f + 0.5f;
    colors[S_HL] = colors[S_H]; /* HLS hue equal to HSV hue */

      }
      else {
    /* The slider that's changed is either H, L, or S. Convert HLS color to RGB and HSV */
    l = colors[S_L] / 255.0f;
    sl = colors[S_SL] / 255.0f;
    h = colors[S_HL] * 360.0f / 255.0f;

    hls_to_rgb(h, l, sl, &r, &g, &b);
    ls_to_sv(l,sl, &s, &v);

    /*
    // Used for testing purposes
    rgb_to_hsv(r, g, b, &h, &s, &v);
    {
    float sv_s, sv_v;
    float _l, _sl;
       text_mode(makecol(255,255,255));
       ls_to_sv(l,sl, &sv_s, &sv_v);
       textprintf(screen,font,352,40,makecol(255,192,255),"%f  %f   ",s,v);
       textprintf(screen,font,352,48,makecol(255,0,255),"%f  %f   ",sv_s,sv_v);

       sv_to_ls(sv_s,sv_v, &_l, &_sl);	// Test to see if inverse brings back same results
       textprintf(screen,font,352,60,makecol(255,192,255),"%f  %f   ",l,sl);
       textprintf(screen,font,352,68,makecol(255,224,255),"%f  %f   ",_l,_sl);
    }
    */


    colors[S_R] = r;
    colors[S_G] = g;
    colors[S_B] = b;

    colors[S_H] = colors[S_HL]; /* HSV hue equal to HLS hue */
    colors[S_SV] = s * 255.0f + 0.5f;
    colors[S_V] = v * 255.0f + 0.5f;

      }


      update_color_bars(r,g,b,h,s,v,l,sl);

      vsync();

      if(bitmap_color_depth(screen)==8) {
    /* set the screen background to the new color */
    RGB rgb;
    rgb.r = colors[S_R]>>2;
    rgb.g = colors[S_G]>>2;
    rgb.b = colors[S_B]>>2;

    set_color(255, &rgb);   /* in 8-bit color modes, we're changing the 'white' color */
      }


      /* Update the rectangle in the middle to the new color */
      update_color_rectangle(r,g,b);
      object_message(&the_dlg[DIALOG_COLOR_BOX], MSG_DRAW, 0);

      /* Display the updated color-bar bitmaps */
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+1], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+2], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+3], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+4], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+5], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+6], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+7], MSG_DRAW, 0);
      object_message(&the_dlg[DIALOG_FIRST_COLOR_BAR+8], MSG_DRAW, 0);


   }

   return D_O_K;
}



/* gui object procedure for the color selection sliders */
int my_slider_proc(int msg, DIALOG *d, int c)
{
   int *color = (int *)d->dp3;

   switch (msg) {

      case MSG_START:
    /* initialise the slider position */
    d->d2 = *color;
    break;

      case MSG_IDLE:
    /* has the slider position changed? */
    if (d->d2 != *color) {
       d->d2 = *color;
       show_mouse(NULL);   // AE: This line in 4.0.3 but not in 4.1.9 or CVS
       object_message(d, MSG_DRAW, 0);
       show_mouse(screen);   // AE: This line in 4.0.3 but not in 4.1.9 or CVS
    }
    break;
   }

   return d_slider_proc(msg, d, c);
}




int main(void)
{
   int i;

   if (allegro_init() != 0)
      return 1;
   install_keyboard();
   install_mouse();
   install_timer();


   /* Set the deepest color depth we can set */
   set_color_depth(32);
   if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
      set_color_depth(24);
      if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
         set_color_depth(16);
         if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
            set_color_depth(15);
            if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
               set_color_depth(8);
               if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
                 allegro_message("Error setting a graphics mode\n%s\n", allegro_error);
                 return 1;
               }
            }
         }
      }
   }


   /* In the case we're using an 8-bit color (256 colors) screen, we must set up the palette */
   if(bitmap_color_depth(screen)==8) {
     PALETTE pal332;
     generate_332_palette(pal332);

      set_palette(pal332);   /* Set the palette to the best approximation of a true-color palette we can get with 8-bit color */

     /* In 8-bit color mode, if there's an RGB table, thie sliders will move a lot more smoothly and the updating will be a lot quicker */
     /* But if there's no RGB table, this has the advantage that the color conversion routines will take into account any changes in the background color */
     /* Instead of changing background color, we could also rely on the colored rectangle like in the other color modes, but using the 3-3-2 palette, this doesn't display the color as accurately as changing the background color */

     /* Create an RGB table to speedup makecol8() */
     /*
      create_rgb_table(&rgb_table, pal332, NULL);
      rgb_map = &rgb_table;
     */
   }

   clear_to_color(screen,makecol(255,255,255));

   /* color the dialog controls appropriately */
   the_dlg[S_R].fg = the_dlg[DIALOG_NUM_SLIDERS+S_R].fg = makecol(255,0,0);   /* R -> Red */
   the_dlg[S_G].fg = the_dlg[DIALOG_NUM_SLIDERS+S_G].fg = makecol(0,255,0);   /* G -> Green */
   the_dlg[S_B].fg = the_dlg[DIALOG_NUM_SLIDERS+S_B].fg = makecol(0,0,255);   /* B -> Blue */

   the_dlg[S_H].fg = the_dlg[DIALOG_NUM_SLIDERS+S_H].fg = makecol(192,192,192);   /* H -> Grey */
   the_dlg[S_SV].fg = the_dlg[DIALOG_NUM_SLIDERS+S_SV].fg = makecol(128,128,128); /* S -> Dark Grey */
   the_dlg[S_V].fg = the_dlg[DIALOG_NUM_SLIDERS+S_V].fg = makecol(0,0,0);         /* V -> Black */

   the_dlg[S_L].fg = the_dlg[DIALOG_NUM_SLIDERS+S_L].fg = makecol(192,192,192);   /* H -> Grey */
   the_dlg[S_SL].fg = the_dlg[DIALOG_NUM_SLIDERS+S_SL].fg = makecol(128,128,128); /* S -> Dark Grey */
   the_dlg[S_HL].fg = the_dlg[DIALOG_NUM_SLIDERS+S_HL].fg = makecol(0,0,0);       /* V -> Black */


   /* Create the bitmaps for the rainbows */
   for(i=0;i<DIALOG_NUM_SLIDERS;i++) {
     if(!(color_bar_bitmap[i] = create_bitmap_ex(32,256,16))) {
      allegro_message("Error creating a color-bar bitmap\n");
      return 1;
    }
    the_dlg[DIALOG_FIRST_COLOR_BAR+i].dp = color_bar_bitmap[i];
   }

   for(i=0;i<DIALOG_NUM_SLIDERS*3;i++) {
     the_dlg[i].bg = makecol(255,255,255);
   }

   the_dlg[DIALOG_COLOR_BOX].fg = makecol(0,0,0);


   text_mode(-1);
   textout(screen,font,"RGB HSV HLS color spaces example.",344,4,makecol(0,0,0));
   textout(screen,font,"Drag sliders to change color values.",344,12,makecol(0,0,0));

   switch(bitmap_color_depth(screen))
   {
      case 32:
      {
         textout(screen,font,"Running in true-color (32-bit 888)",352,24,makecol(128,128,128));
         textout(screen,font,"16777216 colors",352,32,makecol(128,128,128));
      }
      break;

      case 24:
      {
         textout(screen,font,"Running in true-color (24-bit 888)",352,24,makecol(128,128,128));
         textout(screen,font,"16777216 colors",352,32,makecol(128,128,128));
      }
      break;

      case 16:
      {
         textout(screen,font,"Running in hi-color (16-bit 565)",352,24,makecol(128,128,128));
         textout(screen,font,"65536 colors",352,32,makecol(128,128,128));
      }
      break;

      case 15:
      {
         textout(screen,font,"Running in hi-color (15-bit 555)",352,24,makecol(128,128,128));
         textout(screen,font,"32768 colors",352,32,makecol(128,128,128));
      }
      break;

      case 8:
      {
         textout(screen,font,"Running in paletted mode (8-bit 332)",352,24,makecol(128,128,128));
         textout(screen,font,"256 colors",352,32,makecol(128,128,128));
      }
      break;

      default:
      {
         textout(screen,font,"Unknown color depth",400,16,0);
      }
   }


   update_color_rectangle(colors[S_R], colors[S_G], colors[S_B]);
   update_color_bars(colors[S_R], colors[S_G], colors[S_B],
      colors[S_H]*360.0f/255.0f,
	  colors[S_SV]/255.0f, colors[S_V]/255.0f,
      colors[S_L]/255.0f, colors[S_SL]/255.0f);


   do_dialog(the_dlg, -1);


   return 0;
}

END_OF_MAIN();
